
/*----------------------  MYCGI.C ------------------------------*/
/*    Performs calculations passed in via a CGI form string     */
/*    and dispalys the result in HTML, using GIFs of the        */
/*    individual digits.                                        */
/*                                                              */
/*    Requires the keys "first_number", "second_number"         */
/*    and "operation" to be provided by the calling form.       */
/*    Also assumes the presence of 0.GIF...9.GIF, @.GIF and     */
/*    -.GIF in the /images directory of the server root.        */
/*                                                              */
/*    Note: This is intended for demonstration purposes only.   */
/*    A CGI program running on a web server connected to        */
/*    the outside world requires elaborate security precautions */
/*    and at least some degree of fault tolerance.              */
/*                                                              */
/*    This program has neither of these.                        */
/*                                                              */
/*    By running this program, or using this code, you take     */
/*    full responsibility for anything it does.                 */
/*    (Don't sue me, I don't have any money anyway)             */
/*--------------------------------------------------------------*/

#include  <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_BUFFER 2048
#define MAX_LINE     80


char form_data[MAX_BUFFER];
int content_length=0;

/* Eek! Global variables! And a hard-coded buffer, too! It was 5 AM  */
/* and I had a memory leak. Also, this is easier to read.            */
/* Doesn't excuse the globals, but then, who wants to fiddle around  */
/* with prototypes and pointers when the important thing is seeing   */
/* how the interface works?                                          */

void get_form(void)
/* Reads the form data into that nasty hard-coded global buffer */
{
	/* Determine the METHOD */
	if (!strcmp(getenv("REQUEST_METHOD"),"GET"))
	/* remembering that strcmp returns 0 on a match... */
	{
		/* We retrieve the form data from the environment variable */
		strncpy(form_data,getenv("QUERY_STRING"),MAX_BUFFER);
		/* if it's bigger than the buffer, truncate it. */
	}
	else  /* it was POST, so we have to do it the hard way */
	{
		content_length = min(MAX_BUFFER,atoi(getenv("CONTENT_LENGTH")))+1;
		/* The web server sets useful environment variables for you */
		fgets(form_data,content_length,stdin);
		/* which you have to use, because STDIN doesn't go EOF */
		form_data[content_length]='\0'; /* Don't forget to null terminate it! */
	}
}


void find_key(char *key, char *s)
/* You supply the key you want to find (i.e. "first_number") in *key   */
/* and it puts the contents of that key into *s. Crude, but effective  */
{
/* An example string would be first_number=1&operation=Add&second_number=2 */
	char *c, *valstart;
	int length;
	valstart=strstr(form_data,key);  /* find the key */
	valstart=strchr(valstart,'=')+1; /* jump one char past the next '=' */
	/* If you compile this yourself, you may get                        */
	/* Warning: 'Possibly incorrect assignment' here. Just ignore it.   */
	(c=strchr(valstart,'&'))? (length=c-valstart) : (length=strlen(valstart));
	/* Find the end of this value, either by the next '&' or the */
	/* end of the string. */
	length=min(MAX_LINE,length); /* don't overflow that hard-coded buffer! */
	strncpy(s,valstart,length);  /* and pump it into *s */
	s[length]=0; /* Add a \0, so it doesn't fray round the edges */

}

int main(void)
{
	char c,sbuf[MAX_BUFFER];
	int num1,num2,i;
	float result=0; /* If something goes wrong with the form, result=0 */

	get_form();
	/* Tell the web server what's going on... */
	printf("Content-type: text/html\n\n");
	/* Note the two /n characters. A blank line is mandatory here */

	/* and print out the body of the page */
	printf("	<HTML><head><title>CGI demo page</title></head><body background=\"http://localhost/images/bg.gif\">\n");
	printf("	<font color=\"#FF0000\">\n");
	printf("	<center><h1>Using the CGI for trivial purposes</h1>\n");
	printf("	<hr>\n");
	printf("	<form method=get action=\"http://127.0.0.1/cgi-bin/mycgi.exe\">\n");
	/*           escape those quotes ^^                This program^^    */
	printf("	<table width=80%% cols=3> <tr align=center>\n");
	printf("	<td>\n");
	printf("	Take this number<br>\n");
	/* A SELECT element contains a number of OPTION elements */
	printf("	<select name=\"first_number\">\n");
	printf("	<option>1</option> <option>2</option> <option>3</option>\n");
	printf("	<option>4</option> <option>5</option> <option>6</option>\n");
	printf("	<option>7</option> <option>8</option> <option>9</option>\n");
	printf("	</select>\n");
	/* Together, they make a drop-down list */
	printf("	</td>\n");
	printf("	<td>\n");
	printf("	Do this to it<br>\n");
	printf("	<select name=\"operation\">\n");
	printf("	<option>Add</option>      <option>Subtract</option>\n");
	printf("	<option>Multiply</option> <option>Divide</option>\n");
	printf("	</select>\n");
	printf("	</td>\n");
	printf("	<td>\n");
	printf("	With this number<br>\n");
	/* Really, you should pre-load the new form with the original query.  */
	/* You can do this with the SELECTED attribute of the OPTION tag.     */
	/* (Left as an exercise to the reader...)                             */
	printf("	<select name=\"second_number\">\n");
	printf("	<option>1</option> <option>2</option> <option>3</option>\n");
	printf("	<option>4</option> <option>5</option> <option>6</option>\n");
	printf("	<option>7</option> <option>8</option> <option>9</option>\n");
	printf("	</select>\n");
	printf("	</td>\n");
	printf("	</tr></table> <br><br>\n");
	printf("	<input type=submit value=\"The answer, please!\">\n");
	printf("	</form>\n");
	printf("	<br>\n");


	find_key("first_number",sbuf); /* search for key, dump contents  */
											 /* into another hard-coded buffer */
	num1=atoi(sbuf);               /* and take its numerical value   */
	find_key("second_number",sbuf);
	num2=atoi(sbuf);

	find_key("operation",sbuf);
	if(!strcmp(sbuf,"Add"))result=num1+num2;
	if(!strcmp(sbuf,"Subtract"))result=num1-num2;
	if(!strcmp(sbuf,"Multiply"))result=num1*num2;
	if(!strcmp(sbuf,"Divide")&&num2)result=(float)num1/(float)num2;

	sprintf(sbuf,"%.2g",result); /* Re-use that buffer to hold the result */

	/* for each digit... */
	for(i=0;i<strlen(sbuf);i++)
	{
		c=sbuf[i];
		if(c=='.')c='@';/* kludge: you can't call a file '..GIF', so we cheat*/
		printf("<img src=\"http://127.0.0.1/images/%c.gif\">\n",c);
		/* print an image tag for <digit>.GIF in the /images dir */
	}

	printf("</body>\n</html>\n");

	return(0);
}
